unit UnidadMenuPrincipal;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, jpeg, ExtDlgs, ExtCtrls, ComCtrls, shellapi, Buttons;

type
  TformMenuPrincipal = class(TForm)
    Label1: TLabel;
    txtFichero: TEdit;
    dlAbrir: TOpenPictureDialog;
    btSelImagen: TButton;
    imgMiniatura: TImage;
    lInfoMiniatura: TLabel;
    ScrollBox1: TScrollBox;
    imgOriginal: TImage;
    opAutoAjustar: TCheckBox;
    LWEB: TLabel;
    be: TStatusBar;
    btEliminarMiniatura: TBitBtn;
    btGuardarMiniatura: TBitBtn;
    dlGuardar: TSavePictureDialog;
    lInfoTamanoOriginal: TLabel;
    Shape1: TShape;
    Shape2: TShape;
    Shape3: TShape;
    linfoTamanoMiniatura: TLabel;
    procedure btSelImagenClick(Sender: TObject);
    procedure txtFicheroChange(Sender: TObject);
    procedure opAutoAjustarClick(Sender: TObject);
    procedure LWEBClick(Sender: TObject);
    function extraerMiniaturaJPG (ficheroJPG : string) : boolean;
    procedure btGuardarMiniaturaClick(Sender: TObject);
    procedure btEliminarMiniaturaClick(Sender: TObject);

    //para arrastrar y soltar
    procedure SoltarFichero  (var Msg: TWMDropFiles); message WM_DROPFILES;
    procedure FormCreate(Sender: TObject);
    procedure imgOriginalMouseDown(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    procedure MyDragDrop (Sender, Source: TObject; X, Y: Integer);
    procedure MyDragOver (Sender, Source: TObject; X, Y: Integer;
        State: TDragState; var Accept : Boolean);
  public
    { Public declarations }
  end;

var
  formMenuPrincipal: TformMenuPrincipal;
  TheSpot : TPoint;  //para mover imagen con raton en ScrollBox

implementation



{$R *.dfm}

//para drag and drop
procedure TformMenuPrincipal.SoltarFichero (var Msg : TWMDropFiles);
var
  CFileName: array[0..MAX_PATH] of Char;
begin
  try
    if DragQueryFile(Msg.Drop, 0, CFileName, MAX_PATH) > 0 then
    begin
      txtFichero.Text := CFileName;
      Msg.Result := 0;
    end;
  finally
    DragFinish(Msg.Drop); //finaliza arrastrar y soltar
  end;
end;


function tamanoFichero(const fichero : string) : TULargeInteger;
var
  Find: THandle;
  Data: TWin32FindData;
begin
  Result.QuadPart := -1;
  Find := FindFirstFile(PChar(fichero), Data);
  if (Find <> INVALID_HANDLE_VALUE) then
  begin
    Result.LowPart  := Data.nFileSizeLow;
    Result.HighPart := Data.nFileSizeHigh;
    Windows.FindClose(Find);
  end;
end;


procedure eliminarMiniatura (origen, destino : TStream);
var
  Buffer : array[0..$FFFF] of Char;
  Header : array[1..4] of byte;
  tamano : Integer;
begin
  origen.Seek(0, soFromBeginning);
  origen.ReadBuffer(Header, 2);
  if (Header[1] <> $FF) or (Header[2] <> $D8) then
    raise Exception.Create('El identificador de la imagen JPG seleccionada es incorrecto.');
  destino.WriteBuffer(Header, 2);
  repeat
    origen.ReadBuffer(Header, 4);
    tamano := ((Header[3] shl 8) + Header[4]) - 2;

    case Header[2] of

      $FE, $E0..$EF :
        origen.Seek(tamano, soFromCurrent);

      $01,$D0..$D9 :
      begin
        destino.WriteBuffer(Header, 2);
        origen.Seek(-2, soFromCurrent)
      end

      else
      begin
        origen.ReadBuffer(Buffer, tamano);
        destino.WriteBuffer(Header, 4);
        destino.WriteBuffer(Buffer, tamano);
      end;
    end;
  until Header[2] = $DA;
  destino.CopyFrom(origen, origen.Size - origen.Position);
end;



function leerThumbnail (origen, destino : TStream) : Boolean;
var
  Buffer : array[0..$FFFF] of Char;
  Header : array[1..4] of byte;
  tamano : Integer;
  i : Integer;
begin
  Result := false;
  origen.Seek(0, soFromBeginning);
  origen.ReadBuffer(Header, 2);
  if (Header[1] <> $FF) or (Header[2] <> $D8) then
    raise Exception.Create('El identificador de la imagen JPG seleccionada es incorrecto.');
  repeat
    origen.ReadBuffer(Header, 4);
    tamano := ((Header[3] shl 8) + Header[4]) - 2;

    case Header[2] of

      $E1 :
      begin
        origen.ReadBuffer(Buffer, tamano);
        i := 0;
        while i < tamano - 1 do
        begin
          if CompareMem(@Buffer[i], PChar(#$FF#$D8), 2) then
          begin
            destino.WriteBuffer((@Buffer[i])^, tamano - i);
            Result := true;
            break;
          end;
          inc(i);
        end;
        break;
      end;

      $01,$D0..$D9 :
        origen.Seek(-2, soFromCurrent);

      else
        origen.Seek(tamano, soFromCurrent);
    end;
  until (Header[2] = $DA);
end;

function TformMenuPrincipal.extraerMiniaturaJPG (ficheroJPG : string) : boolean;
var
  origen : TFileStream;
  miniaturaJPG : TMemoryStream;
  jpg : TJPEGImage;
begin
  Result := false;
  origen := TFileStream.Create(ficheroJPG, fmOpenRead);
  try
    miniaturaJPG := TMemoryStream.Create;
    try
      if leerThumbnail (origen, miniaturaJPG) then
      begin
        miniaturaJPG.Seek(0, soFromBeginning);
        jpg := TJPEGImage.Create;
        jpg.LoadFromStream(miniaturaJPG);
        imgMiniatura.Picture.Assign(jpg);
        imgMiniatura.Refresh;
        Result := true;
      end;
    finally
      miniaturaJPG.Free;
    end;
  finally
    origen.Free;
  end;
end;

procedure TformMenuPrincipal.btSelImagenClick(Sender: TObject);
begin
  if dlAbrir.Execute then
    txtFichero.Text := dlAbrir.FileName;
end;

procedure TformMenuPrincipal.txtFicheroChange(Sender: TObject);
var
  extraido : boolean;
begin
  if FileExists(txtFichero.Text) then
  begin
    lInfoTamanoOriginal.Caption := 'Tamao: ' +
        FormatFloat('#,### Bytes', tamanoFichero (txtFichero.Text).QuadPart) +
        ' (' + FormatFloat('###,###,###,##0.00 KB',
            tamanoFichero (txtFichero.Text).QuadPart / 1024) + ')';

    imgOriginal.Picture.LoadFromFile(txtFichero.Text);
    extraido := extraerMiniaturaJPG (txtFichero.Text);
    if not extraido then
    begin
      be.Panels[0].Text := 'No se ha encontrado miniatura en el fichero de imagen seleccionado';
      imgMiniatura.Picture := nil;
      lInfoMiniatura.Caption := '<< Sin miniatura';
      imgMiniatura.Visible := false;
      btEliminarMiniatura.Enabled := false;
      btGuardarMiniatura.Enabled := false;
    end
    else
    begin
      lInfoMiniatura.Caption := 'Miniatura >>';
      imgMiniatura.Visible := true;
      be.Panels[0].Text := 'Se ha extrado una miniatura (thumbnail) de la imagen seleccionada';
      btEliminarMiniatura.Enabled := true;
      btGuardarMiniatura.Enabled := true;
    end;
  end
  else
  begin
    be.Panels[0].Text := 'No existe el fichero de imagen JPEG seleccionado';
    imgMiniatura.Visible := false;
  end;
end;

procedure TformMenuPrincipal.opAutoAjustarClick(Sender: TObject);
begin
  if opAutoAjustar.Checked then
  begin
    imgOriginal.AutoSize := false;
    imgOriginal.Stretch := true;
    imgOriginal.Proportional := true;
    imgOriginal.Height := ScrollBox1.Height - 10;
    imgOriginal.Width := ScrollBox1.Width - 10;
  end
  else
  begin
    imgOriginal.AutoSize := true;
    imgOriginal.Stretch := false;
    imgOriginal.Proportional := false;
  end;                                
  imgOriginal.Refresh;
  imgOriginal.Repaint;
end;

procedure TformMenuPrincipal.LWEBClick(Sender: TObject);
begin
  ShellExecute(Handle, Nil, PChar('http://www.ajpdsoft.com'),
      Nil, Nil, SW_SHOWNORMAL);
end;

procedure TformMenuPrincipal.btGuardarMiniaturaClick(Sender: TObject);
begin
  dlGuardar.FileName :=
      IncludeTrailingPathDelimiter (ExtractFilePath (txtFichero.Text)) +
          ChangeFileExt (ExtractFileName(txtFichero.Text),'') +
          '_thumbnail' + ExtractFileExt(txtFichero.Text);
  if dlGuardar.Execute then
    imgMiniatura.Picture.SaveToFile(dlGuardar.FileName);
end;

procedure TformMenuPrincipal.btEliminarMiniaturaClick(Sender: TObject);
var
  origen, destino : TFileStream;
begin
  MessageDlg('Antes de eliminar la miniatura de la imagen JPG es ' +
      'recomendable que no reemplace el fichero original. Por ello ' +
      'se le solicitar el destino de la imagen nueva igual que ' +
      'la original pero sin la miniatura.' + chr(13) + chr(13) +
      'Una vez que haya verificado que la imagen sin miniatura es correcta '+
      'podr eliminar la imagen JPG original con la miniatura.',
      mtInformation, [mbok], 0);
  dlGuardar.Title := 'Guardar imagen sin miniatura';
  dlGuardar.FileName := ChangeFileExt(txtFichero.Text, '_sin_miniatura.jpg');
  if dlGuardar.Execute then
  begin
    origen := TFileStream.Create(txtFichero.Text, fmOpenRead);
    try
      destino := TFileStream.Create(dlGuardar.FileName, fmCreate);
      try
        eliminarMiniatura (origen, destino);
      finally
        destino.Free;
      end;
    finally
      origen.Free;
    end;
  end;
end;

procedure TformMenuPrincipal.MyDragOver (Sender, Source: TObject;
    X, Y: Integer; State: TDragState; var Accept : Boolean);
begin
  //para mover imagen con ratn en ScrollBox
  Accept := (Source is TImage);
end;

procedure TformMenuPrincipal.MyDragDrop
  (Sender, Source: TObject; X, Y: Integer);
begin
  //para mover imagen en ScrollBox con ratn
  if Source is TImage then
  begin
    if Sender is TImage then
    begin
      TImage(Source).Left := TImage(Source).Left + x - TheSpot.X;
      TImage(Source).Top := TImage(Source).Top + y  - TheSpot.Y;
    end
    else
    begin
      TImage(Source).Left := X - TheSpot.X;
      TImage(Source).Top := Y - TheSpot.Y;
    end;
  end;
end;


procedure TformMenuPrincipal.FormCreate(Sender: TObject);
begin
  //permitimos arrastrar y soltar ficheros externos
  DragAcceptFiles(Handle, True);

  imgOriginal.OnDragOver := MyDragOver;
  imgOriginal.OnDragDrop := MyDragDrop;

  ScrollBox1.OnDragOver := MyDragOver;
  ScrollBox1.OnDragDrop := MyDragDrop;
end;

procedure TformMenuPrincipal.imgOriginalMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  //para mover imagen con ratn en ScrollBox
  TheSpot.X := X;
  TheSpot.Y := Y;
  imgOriginal.BeginDrag(True);
end;

end.
